LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;

ENTITY BF_II IS
  GENERIC(
    Data_width : integer := 36
    );
  PORT(
    -- clk      : IN     std_logic;
    --   rst_n 	  : IN		   std_logic;
    T        : IN     std_logic;
    S        : IN     std_logic;
    BF2_I1   : IN     std_logic_vector (Data_width-1 DOWNTO 0);
    BF2_I2   : IN     std_logic_vector (Data_width-1 DOWNTO 0);
    BF2_O1  : OUT    std_logic_vector (Data_width-1 DOWNTO 0);
    BF2_O2  : OUT    std_logic_vector (Data_width-1 DOWNTO 0)
    );
  
END BF_II;

ARCHITECTURE rtl OF BF_II IS
  signal BF2_I1_re:     std_logic_vector (Data_width/2-1 downto 0);
  signal BF2_I1_imag:   std_logic_vector (Data_width/2-1 downto 0);
  signal BF2_I2_re:     std_logic_vector (Data_width/2-1 downto 0);
  signal BF2_I2_imag:   std_logic_vector (Data_width/2-1 downto 0);
  
  signal BF2_O1_re:    std_logic_vector (Data_width/2  downto 0);
  signal BF2_O1_imag:  std_logic_vector (Data_width/2  downto 0);
  signal BF2_O2_re:    std_logic_vector (Data_width/2  downto 0);
  signal BF2_O2_imag:  std_logic_vector (Data_width/2  downto 0);
  
  signal add1_i1:  std_logic_vector (Data_width/2-1  downto 0);
  signal add1_i2:  std_logic_vector (Data_width/2-1  downto 0);
  signal add3_i1:  std_logic_vector (Data_width/2-1  downto 0);
  signal add3_i2:  std_logic_vector (Data_width/2-1  downto 0);
  
  signal D:        std_logic;
begin
  D <= S and T;
  
  
  
  BF2_I1_re      <= BF2_I1 (Data_width-1 downto Data_width/2);
  BF2_I1_imag    <= BF2_I1 (Data_width/2-1 downto 0);
  
  
  --Switching the 2nd input
  BF2_I2_re   <= BF2_I2 (Data_width-1 downto Data_width/2) when D ='0' else BF2_I2 (Data_width/2-1 downto 0);
  BF2_I2_imag <= BF2_I2 (Data_width/2-1 downto 0) when D ='0' else BF2_I2 (Data_width-1 downto Data_width/2);    
  
  
  add1_i1   <= (BF2_I1_imag );     --when D='0' else BF2_I1_imag ;
  add1_i2   <= (BF2_I2_imag)     when D='0' else not(BF2_I2_imag) ;
  add3_i1   <= (BF2_I1_imag );    -- when D='0' else BF2_I1_imag ;
  add3_i2   <= not((BF2_I2_imag ))    when D='0' else BF2_I2_imag;
  
  BF2_O1_re <= (BF2_I1_re(17) & BF2_I1_re)  + (BF2_I2_re(17) & BF2_I2_re);
  BF2_O2_re <= (BF2_I1_re(17) & BF2_I1_re)  - (BF2_I2_re(17) & BF2_I2_re );
  BF2_O1_imag <= (add1_i1(17) & add1_i1) + (add1_i2(17) & add1_i2);
  BF2_O2_imag <= (add3_i1(17) & add3_i1) + (add3_i2(17) & add3_i2);
  
  
  
  
  
  --Data 1
  BF2_O1 <= BF2_I1 when S ='0' else (BF2_O1_re(Data_width/2 downto 1) & BF2_O1_imag(Data_width/2 downto 1)); 
  --Data 2
  BF2_O2 <= BF2_I2 when S ='0' else (BF2_O2_re(Data_width/2 downto 1) & BF2_O2_imag(Data_width/2 downto 1));

END; 
