LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;


entity BF_I_FIFO is
  
  generic (
    Fifo_size  : integer := -1;
    Data_width : integer := 36;
    Sindex     : integer := -1;
    Memtype    : integer := -1);

  port (
    clk      : in  std_logic;
    ctrl     : in std_logic_vector(12 downto 0);
    ctrl_out : out std_logic_vector(12 downto 0);
    Data_in  : in  std_logic_vector(Data_width-1 downto 0);
    Data_out : out std_logic_vector(Data_width -1 downto 0));
  

end BF_I_FIFO;


architecture rtl of BF_I_FIFO is
  signal coeff: std_logic_vector(23 downto 0);
  signal mult_result, tofifo, fromfifo : std_logic_vector(Data_width-1 downto 0);

  signal address : std_logic_vector(7 downto 0);
begin  -- rtl

  ctrl_out <= ctrl;
  
  -- Select one of the three coefficient memories depending on which module
  -- that is actually instantiated. Also create an address generator with
  -- appropriate parameters.
  MEMM: if Memtype = 1 generate
    U_MEM: entity work.Memory24M_256
      port map  (       	 
        address => address,
        coeff     => coeff);

    U_ADDRGEN: entity work.address_generator
      generic map(
        Comparator_value => "11000000")
      port map(
        clk => clk,
        ctrl => ctrl,
        address => address);
  end generate MEMM;


  
  MEMS: if Memtype = 2 generate
    U_MEM: entity work.Memory24S_256
      port map  (       	 
        address => address,
        coeff     => coeff);

    U_ADDRGEN: entity work.address_generator
      generic map(
        Comparator_value => "11110000")
      port map(
        clk => clk,
        ctrl => ctrl,
        address => address);
  end generate MEMS;

  
  MEMT: if Memtype = 3 generate
    U_MEM: entity work.Memory24T_256
      port map  (       	 
        address => address,
        coeff     => coeff);

    U_ADDRGEN: entity work.address_generator
      generic map(
        Comparator_value => "11111100")
      port map(
        clk => clk,
        ctrl => ctrl,
        address => address);
  end generate MEMT;


  
  U_MULT: entity work.Mult_complex
    port map  (
      -- clk =>  clk,
      mult_i1 => Data_in,
      mult_i2 => coeff,
      mult_o  => mult_result
      );


  U_BF:  entity work.BF_I
    port map (
      S       => ctrl(Sindex+1),
      BF1_I1  => fromfifo,
      BF1_I2  => mult_result,
      BF1_O1 => Data_out,
      BF1_O2 => tofifo
      );

  U_FIFO:  entity work.FIFO
    generic map (
      Fifo_size => Fifo_size)
    port map (
      clk =>  clk,
      flush => ctrl(0),
      Data_in  => tofifo,
      Data_out => fromfifo
      );

end rtl;
