library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity control is
  generic (counter_range : integer := 8);
  port(
    clk       : in  std_logic;
    rst       : in  std_logic;
    ctrl : out std_logic_vector(12 downto 0)
    );
end control;

architecture rtl of control is
  signal t1_en, t2_en, t3_en, t4_en : std_logic;
  signal t1_en_reg, t2_en_reg, t3_en_reg, t4_en_reg : std_logic;

  signal test_1 : std_logic_vector(7 downto 0);
  signal test_2 : std_logic_vector(5 downto 0);
  signal test_3 : std_logic_vector(3 downto 0);
  signal test_4 : std_logic_vector(1 downto 0);

  signal Q : std_logic;

  signal counter   : std_logic_vector(1 downto 0);
  signal S_counter : std_logic_vector(7 downto 0);  -- Switches for Butterfly

  signal T : std_logic_vector(3 downto 0);

begin
  ctrl <= T & S_counter & rst;


  start : process(clk)
  begin
    if rising_edge(clk) then
      if counter = "00" then
        Q <= '1';
      end if;
      counter <= counter+1;

      if rst = '1' then
        counter <= (others => '0');
        Q       <= '0';
      end if;
    end if;
  end process;

  count : process(clk)
  begin
    if rising_edge(clk) then
      if Q = '1' then
        S_counter <= S_counter+1;
      end if;
      if rst = '1' then
        S_counter <= (others => '0');
      end if;
    end if;
  end process;


  t1_en <= '1' when (S_counter(7) = '1') or (t1_en_reg = '1') else '0';

  count1 : process(clk)
  begin
    if rising_edge(clk) then
      t1_en_reg <= t1_en;
      if t1_en = '1' then
        test_1 <= test_1+1;
      end if;
      if rst = '1' then
        test_1    <= (others => '0');
        t1_en_reg <= '0';
      end if;
    end if;
  end process;

  forT0 : process(clk)
  begin
    if rising_edge(clk) then
      if test_1 = "10111111" then
        T(0) <= '1';
      elsif test_1 = "11111111" then
        T(0) <= '0';
      end if;
      if (rst = '1') then
        T(0) <= '0';
      end if;
    end if;
  end process;

  t2_en <= '1' when (S_counter(7 downto 5) = "111")  or (t2_en_reg = '1') else '0';

  count2 : process(clk)
  begin
    if rising_edge(clk) then
      t2_en_reg <= t2_en;
      if t2_en = '1' then
        test_2 <= test_2+1;
      end if;
      if rst = '1' then
        test_2    <= (others => '0');
        t2_en_reg <= '0';
      end if;
    end if;
  end process;

  forT1 : process(clk)
  begin
    if rising_edge(clk) then
      if test_2 = "101111" then
        T(1) <= '1';
      elsif test_2 = "111111" then
        T(1) <= '0';
      end if;
      if (rst = '1') then
        T(1) <= '0';
      end if;
    end if;
  end process;



  t3_en <= '1' when (S_counter(7 downto 3) = "11111") or (t3_en_reg = '1') else '0';

  count3 : process(clk)
  begin
    if rising_edge(clk) then
      t3_en_reg <= t3_en;
      if t3_en = '1' then
        test_3 <= test_3+1;
      end if;
      if rst = '1' then
        test_3    <= (others => '0');
        t3_en_reg <= '0';
      end if;
    end if;
  end process;

  forT2 : process(clk)
  begin
    if rising_edge(clk) then
      if test_3 = "1011" then
        T(2) <= '1';
      elsif test_3 = "1111" then
        T(2) <= '0';
      end if;
      if (rst = '1') then
        T(2) <= '0';
      end if;
    end if;
  end process;

  t4_en <= '1' when (S_counter(7 downto 1) = "1111111")  or (t4_en_reg = '1');

  count4 : process(clk)
  begin
    if rising_edge(clk) then
      t4_en_reg <= t4_en;
      if t4_en = '1' then
        test_4 <= test_4+1;
      end if;
      if rst = '1' then
        test_4    <= (others => '0');
        t4_en_reg <= '0';
      end if;
    end if;
  end process;
  forT3 : process(clk)
  begin
    if rising_edge(clk) then
      if test_4 = "10" then
        T(3) <= '1';
      elsif test_4 = "11" then
        T(3) <= '0';
      end if;
      if (rst = '1') then
        T(3) <= '0';
      end if;
    end if;
  end process;



end;
