LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;

ENTITY fft_processor IS
  GENERIC(  Data_width : integer :=36  );
  port(
    clk    : in   std_logic;
    rst    : in   std_logic;
    Input  : in   std_logic_vector (Data_width-1 downto 0);
    Output : out  std_logic_vector (Data_width-1 downto 0)
    );
end fft_processor;

architecture rtl of fft_processor is
  signal ctrl, ctrl_l1, ctrl_l2, ctrl_l3, ctrl_l4  : std_logic_vector(12 downto 0);
  signal ctrl_l5, ctrl_l6, ctrl_l7 : std_logic_vector(12 downto 0);

  signal data_l7, data_l6, data_l5 : std_logic_vector (Data_width - 1 downto 0);
  signal data_l4, data_l3, data_l2 : std_logic_vector (Data_width - 1 downto 0);
  signal data_l1 : std_logic_vector (Data_width - 1 downto 0);

  signal layer1_tofifo, layer1_fromfifo : std_logic_vector (Data_width - 1 downto 0);
  

begin

  -- First BF_I is a special case since it doesn't have a multiplier
  U_BF_I_layer1  : entity work.BF_I
    port map (
      S       => ctrl(8),
      BF1_I1  => layer1_fromfifo,
      BF1_I2  => Input,
      BF1_O1 => data_l1,
      BF1_O2 => layer1_tofifo
      );
  U_FIFO_layer1 : entity work.FIFO
    generic map (
      Fifo_size => 128)
    port map (
      clk =>  clk,
      flush => ctrl(0),
      Data_in  => layer1_tofifo,
      Data_out => layer1_fromfifo
      );
  ctrl_l1 <= ctrl;

  U_layer2 : entity work.BF_II_FIFO
    generic map (
      Tindex    => 0,
      Sindex    => 6,
      Fifo_size => 64)
    port map (
      clk      => clk,
      ctrl     => ctrl_l1,
      ctrl_out => ctrl_l2,
      Data_in  => data_l1,
      Data_out => data_l2);

  U_layer3: entity work. BF_I_FIFO
    generic map (
      Sindex => 5,
      Fifo_size => 32,
      Memtype => 1)
    port map (
      clk      => clk,
      ctrl => ctrl_l2,
      ctrl_out => ctrl_l3,
      Data_in  => data_l2,
      Data_out => data_l3);
      
  U_layer4 : entity work.BF_II_FIFO
    generic map (
      Tindex    => 1,
      Sindex    => 4,
      Fifo_size => 16)
    port map (
      clk      => clk,
      ctrl     => ctrl_l3,
      ctrl_out => ctrl_l4,
      Data_in  => data_l3,
      Data_out => data_l4);

  
  U_layer5: entity work. BF_I_FIFO
    generic map (
      Sindex => 3,
      Fifo_size => 8,
      Memtype => 2)
    port map (
      clk      => clk,
      ctrl => ctrl_l4,
      ctrl_out    => ctrl_l5,
      Data_in  => data_l4,
      Data_out => data_l5);

  U_layer6 : entity work.BF_II_FIFO
    generic map (
      Tindex    => 2,
      Sindex    => 2,
      Fifo_size => 4)
    port map (
      clk      => clk,
      ctrl     => ctrl_l5,
      ctrl_out => ctrl_l6,
      Data_in  => data_l5,
      Data_out => data_l6);

  U_layer7 : entity work. BF_I_FIFO
    generic map (
      Sindex => 1,
      Fifo_size => 2,
      Memtype => 3)
    port map (
      clk      => clk,
      ctrl     => ctrl_l6,
      ctrl_out => ctrl_l7,
      Data_in  => data_l6,
      Data_out => data_l7);


  U_layer8 : entity work.BF_II_FIFO
    generic map (
      Tindex    => 3,
      Sindex    => 0,
      Fifo_size => 1)
    port map (
      clk      => clk,
      ctrl     => ctrl_l7,
      ctrl_out => open,
      Data_in  => data_l7,
      Data_out => Output);
    
  
  U_CTRL:entity work.Control
    port map (
      clk =>  clk,
      rst => rst,
      ctrl => ctrl
      );    
End;  


